@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2023, Intel Corporation
+ * Copyright (c) 2016-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,7 @@
 #ifndef PT_BLOCK_DECODER_H
 #define PT_BLOCK_DECODER_H
 
-#include "pt_query_decoder.h"
+#include "pt_event_decoder.h"
 #include "pt_image.h"
 #include "pt_retstack.h"
 #include "pt_ild.h"
@@ -44,8 +44,8 @@
  * of trace.
  */
 struct pt_block_decoder {
-	/* The Intel(R) Processor Trace query decoder. */
-	struct pt_query_decoder query;
+	/* The event decoder. */
+	struct pt_event_decoder evdec;
 
 	/* The configuration flags.
 	 *
@@ -66,7 +66,17 @@ struct pt_block_decoder {
 	/* The current address space. */
 	struct pt_asid asid;
 
-	/* The current Intel(R) Processor Trace event. */
+	/* The current event.
+	 *
+	 * This will be valid as long as there are events available, i.e. until
+	 * @status is not negative.
+	 *
+	 * The decoder starts by reading the first event after synchronizing
+	 * onto the trace stream.
+	 *
+	 * When it is done processing an event, it fetches the next event for
+	 * the next iteration.
+	 */
 	struct pt_event event;
 
 	/* The call/return stack for ret compression. */
@@ -86,14 +96,22 @@ struct pt_block_decoder {
 	 */
 	uint64_t ip;
 
+	/* The last TSC.
+	 *
+	 * We use it to check for time updates when generating tick events.
+	 */
+	uint64_t tsc;
+
+	/* The number of lost MTC and CYC packets. */
+	uint32_t lost_mtc, lost_cyc;
+
+	/* The last CBR. */
+	uint32_t cbr;
+
 	/* The current execution mode. */
 	enum pt_exec_mode mode;
 
-	/* The status of the last successful decoder query.
-	 *
-	 * Errors are reported directly; the status is always a non-negative
-	 * pt_status_flag bit-vector.
-	 */
+	/* The last status of the event decoder. */
 	int status;
 
 	/* A collection of flags defining how to proceed flow reconstruction:
@@ -102,12 +120,12 @@ struct pt_block_decoder {
 	 */
 	uint32_t enabled:1;
 
-	/* - process @event. */
-	uint32_t process_event:1;
-
 	/* - instructions are executed speculatively. */
 	uint32_t speculative:1;
 
+	/* - whether @tsc tracks wall-clock time. */
+	uint32_t has_tsc:1;
+
 	/* - process @insn/@iext.
 	 *
 	 *   We have started processing events binding to @insn/@iext.  The
@@ -127,6 +145,15 @@ struct pt_block_decoder {
 
 	/* - a ptwrite event has already been bound to @insn/@iext. */
 	uint32_t bound_ptwrite:1;
+
+	/* - an iret event has already been bound to @insn/@iext. */
+	uint32_t bound_iret:1;
+
+	/* - a vmentry event has already been bound to @insn/@iext. */
+	uint32_t bound_vmentry:1;
+
+	/* - an uiret event has already been bound to @insn/@iext. */
+	uint32_t bound_uiret:1;
 };
 
 
@@ -141,4 +168,21 @@ extern int pt_blk_decoder_init(struct pt_block_decoder *decoder,
 /* Finalize a block decoder. */
 extern void pt_blk_decoder_fini(struct pt_block_decoder *decoder);
 
+static inline const struct pt_config *
+pt_blk_config(const struct pt_block_decoder *decoder)
+{
+	if (!decoder)
+		return NULL;
+
+	return pt_evt_config(&decoder->evdec);
+}
+
+static inline const uint8_t *pt_blk_pos(const struct pt_block_decoder *decoder)
+{
+	if (!decoder)
+		return NULL;
+
+	return pt_evt_pos(&decoder->evdec);
+}
+
 #endif /* PT_BLOCK_DECODER_H */
