@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2023, Intel Corporation
+ * Copyright (c) 2016-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -171,6 +171,28 @@ extern int pt_insn_binds_to_vmcs(const struct pt_insn *insn,
 extern int pt_insn_is_ptwrite(const struct pt_insn *insn,
 			      const struct pt_insn_ext *iext);
 
+/* Check if the instruction @insn/@iext is a return-from-interrupt instruction.
+ *
+ * Returns non-zero if it is, zero if it isn't (or @insn/@iext is NULL).
+ */
+extern int pt_insn_is_iret(const struct pt_insn *insn,
+			   const struct pt_insn_ext *iext);
+
+/* Check if the instruction @insn/@iext is a vmentry instruction.
+ *
+ * Returns non-zero if it is, zero if it isn't (or @insn/@iext is NULL).
+ */
+extern int pt_insn_is_vmentry(const struct pt_insn *insn,
+			      const struct pt_insn_ext *iext);
+
+/* Check if the instruction @insn/@iext is a return-from-user-interrupt
+ * instruction.
+ *
+ * Returns non-zero if it is, zero if it isn't (or @insn/@iext is NULL).
+ */
+extern int pt_insn_is_uiret(const struct pt_insn *insn,
+			    const struct pt_insn_ext *iext);
+
 /* Determine the IP of the next instruction.
  *
  * Tries to determine the IP of the next instruction without using trace and
