@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2023, Intel Corporation
+ * Copyright (c) 2013-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,11 @@
 #include <stdint.h>
 
 #if defined(FEATURE_THREADS)
-#  include <threads.h>
+#  if !defined(__STDC_NO_THREADS__)
+#    include <threads.h>
+#  else
+#    include "pt_threads.h"
+#  endif
 #endif /* defined(FEATURE_THREADS) */
 
 struct pt_section;
