@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2023, Intel Corporation
+ * Copyright (c) 2013-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,7 +28,6 @@
  */
 
 #include "pt_cpu.h"
-#include "pt_cpuid.h"
 
 #include "intel-pt.h"
 
@@ -36,60 +35,6 @@
 #include <stdlib.h>
 
 
-static const char * const cpu_vendors[] = {
-	"",
-	"GenuineIntel"
-};
-
-enum {
-	pt_cpuid_vendor_size = 12
-};
-
-union cpu_vendor {
-	/* The raw data returned from cpuid. */
-	struct {
-		uint32_t ebx;
-		uint32_t edx;
-		uint32_t ecx;
-	} cpuid;
-
-	/* The resulting vendor string. */
-	char vendor_string[pt_cpuid_vendor_size];
-};
-
-static enum pt_cpu_vendor cpu_vendor(void)
-{
-	union cpu_vendor vendor;
-	uint32_t eax;
-	size_t i;
-
-	memset(&vendor, 0, sizeof(vendor));
-	eax = 0;
-
-	pt_cpuid(0u, &eax, &vendor.cpuid.ebx, &vendor.cpuid.ecx,
-		 &vendor.cpuid.edx);
-
-	for (i = 0; i < sizeof(cpu_vendors)/sizeof(*cpu_vendors); i++)
-		if (strncmp(vendor.vendor_string,
-			    cpu_vendors[i], pt_cpuid_vendor_size) == 0)
-			return (enum pt_cpu_vendor) i;
-
-	return pcv_unknown;
-}
-
-static uint32_t cpu_info(void)
-{
-	uint32_t eax, ebx, ecx, edx;
-
-	eax = 0;
-	ebx = 0;
-	ecx = 0;
-	edx = 0;
-	pt_cpuid(1u, &eax, &ebx, &ecx, &edx);
-
-	return eax;
-}
-
 int pt_cpu_parse(struct pt_cpu *cpu, const char *s)
 {
 	const char sep = '/';
@@ -138,28 +83,3 @@ int pt_cpu_parse(struct pt_cpu *cpu, const char *s)
 
 	return 0;
 }
-
-int pt_cpu_read(struct pt_cpu *cpu)
-{
-	uint32_t info;
-	uint16_t family;
-
-	if (!cpu)
-		return -pte_invalid;
-
-	cpu->vendor = cpu_vendor();
-
-	info = cpu_info();
-
-	cpu->family = family = (info>>8) & 0xf;
-	if (family == 0xf)
-		cpu->family += (info>>20) & 0xf;
-
-	cpu->model = (info>>4) & 0xf;
-	if (family == 0x6 || family == 0xf)
-		cpu->model += (info>>12) & 0xf0;
-
-	cpu->stepping = (info>>0) & 0xf;
-
-	return 0;
-}
