@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2023, Intel Corporation
+ * Copyright (c) 2013-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -97,6 +97,11 @@ struct sb_filelist {
 
 #endif /* defined(FEATURE_SIDEBAND) */
 
+enum {
+	/* The maximal size of a label in characters. */
+	l_max	= 256
+};
+
 /* Represents the parser.  */
 struct parser {
 	/* File pointer to the trace output file.  */
