@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2023, Intel Corporation
+ * Copyright (c) 2017-2025, Intel Corporation
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,6 @@
 
 #include "libipt-sb.h"
 
-#include "intel-pt.h"
-
 
 #ifndef FEATURE_PEVENT
 
@@ -353,6 +351,11 @@ int pt_sb_pevent_init(struct pt_sb_pevent_priv *priv,
 	priv->tsc_offset = config->tsc_offset;
 	priv->location = ploc_unknown;
 
+#if (LIBIPT_SB_VERSION >= 0x201)
+	if (pev_config_has(config, sample_config))
+		priv->pev.sample_config = config->sample_config;
+#endif
+
 	return 0;
 }
 
@@ -881,12 +884,10 @@ static int pt_sb_pevent_print(struct pt_sb_pevent_priv *priv, FILE *stream,
 	if (errcode < 0)
 		return errcode;
 
-	/* Print samples if configured. */
-	if (priv->pev.sample_type) {
-		errcode = pt_sb_pevent_print_samples(event, stream, flags);
-		if (errcode < 0)
-			return errcode;
-	}
+	/* Print samples that were configured for the record. */
+	errcode = pt_sb_pevent_print_samples(event, stream, flags);
+	if (errcode < 0)
+		return errcode;
 
 	if (flags)
 		fprintf(stream, "\n");
@@ -1540,6 +1541,88 @@ static int ploc_from_event(enum pt_sb_pevent_loc *loc,
 					    event->variant.tick.ip);
 
 		break;
+
+	case ptev_tip:
+		return ploc_from_ip(loc, priv, event->variant.tip.ip);
+
+#if (LIBIPT_VERSION >= 0x201)
+	case ptev_iflags:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.iflags.ip);
+
+		break;
+
+	case ptev_interrupt:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.interrupt.ip);
+
+		break;
+
+	case ptev_iret:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.iret.ip);
+
+		break;
+
+	case ptev_smi:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.smi.ip);
+
+		break;
+
+	case ptev_rsm:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.rsm.ip);
+
+		break;
+
+	case ptev_init:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.init.ip);
+
+		break;
+
+	case ptev_vmentry:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.vmentry.ip);
+
+		break;
+
+	case ptev_vmexit:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.vmexit.ip);
+
+		break;
+
+	case ptev_shutdown:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.shutdown.ip);
+
+		break;
+
+	case ptev_uintr:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.uintr.ip);
+
+		break;
+
+	case ptev_uiret:
+		if (!event->ip_suppressed)
+			return ploc_from_ip(loc, priv,
+					    event->variant.uiret.ip);
+
+		break;
+#endif
 	}
 
 	*loc = ploc_unknown;
